/***
*
* BNBT Beta 7.7 - A C++ BitTorrent Tracker
* Copyright (C) 2003 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#include <sstream>
#include "atom.hh"

//
// CAtomInt
//

std::string CAtomInt :: bencode( ) const
{
	std::stringstream s;
	s << 'i' << getValue() << 'e';
	return s.str();
}

//
// CAtomString
//

std::string CAtomString :: bencode( ) const
{
    std::stringstream s;
    s << m_strString.size() << ':' << m_strString;
    return s.str();
}

//
// CAtomList
//

std::string CAtomList :: bencode( ) const
{
    std::stringstream s;
    s << 'l';
    for(unsigned a=0; a<m_vecList.size(); ++a)
        s << m_vecList[a]->bencode();
    s << 'e';
	return s.str();
}

void CAtomList :: Randomize( )
{
	std::random_shuffle( m_vecList.begin( ), m_vecList.end( ) );
}

void CAtomList :: setValue(const std::vector<autoptr<CAtom > >& vecList )
{
	m_vecList.resize(vecList.size());
	for(unsigned a=0; a<vecList.size(); ++a)
	    m_vecList[a] = vecList[a]->clone();
}

void CAtomList :: delItem( autoptr<CAtom>atmItem )
{
	for( std::vector<autoptr<CAtom> > :: iterator
	     i = m_vecList.begin( ); i != m_vecList.end( ); i++ )
	{
		if( &**i == &*atmItem )
		{
			m_vecList.erase( i );
			return;
		}
	}
}

void CAtomList :: addItem( autoptr<CAtom>atmItem )
{
	m_vecList.push_back( atmItem );
}

//
// CAtomDict
//

void CAtomDict :: setValue(const std::map<std::string, autoptr<CAtom> >& mapDict )
{
	for( std::map<std::string, autoptr<CAtom> > :: const_iterator
	     i = mapDict.begin( ); i != mapDict.end( ); i++ )
	    setItem(i->first, i->second->clone());
}

std::string CAtomDict :: bencode( ) const
{
    std::stringstream s;
    s << 'd';
	for( std::map<std::string, autoptr<CAtom> > :: const_iterator
	     i = m_mapDict.begin( ); i != m_mapDict.end( ); i++ )
	{
	    s << CAtomString(i->first).bencode();
	    s << i->second->bencode();
	}
    s << 'e';
	return s.str();
}

void CAtomDict :: delItem( const std::string& strKey )
{
	m_mapDict.erase( strKey );
}

autoptr<CAtom>CAtomDict :: getItem( const std::string& strKey, autoptr<CAtom> ret) const
{
	std::map<std::string, autoptr<CAtom> > :: const_iterator i = m_mapDict.find( strKey );
	if( i != m_mapDict.end( ) ) return i->second;
	return ret;
}

void CAtomDict :: setItem( const std::string& strKey, autoptr<CAtom>pValue )
{
    m_mapDict[strKey] = pValue;
}

CAtomInt CAtomDict::getItemInt(const std::string& strKey, CAtomInt def) const
{
    autoptr<CAtom> a = getItem(strKey); if(!a) return def;
    const CAtomInt* res = a->get_int(); if(!res) return def;
    return *res;
}

CAtomString CAtomDict::getItemStr(const std::string& strKey, CAtomString def) const
{
    autoptr<CAtom> a = getItem(strKey); if(!a) return def;
    const CAtomString* res = a->get_str(); if(!res) return def;
    return *res;
}

CAtomList CAtomDict::getItemList(const std::string& strKey, CAtomList def) const
{
    autoptr<CAtom> a = getItem(strKey); if(!a) return def;
    const CAtomList* res = a->get_list(); if(!res) return def;
    return *res;
}

CAtomDict CAtomDict::getItemDict(const std::string& strKey, CAtomDict def) const
{
    autoptr<CAtom> a = getItem(strKey); if(!a) return def;
    const CAtomDict* res = a->get_dict(); if(!res) return def;
    return *res;
}

const CAtomInt* CAtom::get_int() const { return dynamic_cast<const CAtomInt*> (this); }
const CAtomList* CAtom::get_list() const { return dynamic_cast<const CAtomList*> (this); }
const CAtomDict* CAtom::get_dict() const { return dynamic_cast<const CAtomDict*> (this); }
const CAtomString* CAtom::get_str() const { return dynamic_cast<const CAtomString*> (this); }
